// Complete testing of Tvector class

#include <iostream.h>
#include "tvector.h"


// Iteration function for printing with for_each
inline void dprint (const double &d, void *extra)
   {
   cout << d << ",";
   }
// Function for testing first_that
inline int more_than_5 (const double &a, void *extra)
   {
   if(a>5) return 1;
   else return 0;
   }
// Function for testing last_that and showing how to pass arguments
inline int less_than (const double &a, void *extra)
   {
   double arg = *((double*)extra);

   if(a<arg) return 1;
   else return 0;
   }

int main (void)
   {
   cout << "\n\nVerifying Tvector class operation...\n";

   int i, num;

   // The vector to test
   Tvector<double> a;

   // Making sure we start with an empty vector
   cout << "Verifying initial size\n";

   num = a.size();
   if(num==0) cout << "Initial size verified\n";
   else cout << "Failure in initial size check!\n";

   // Resizing and checking
   cout << "Resizing to 10\n";

   a.resize(10);
   for(i=0;i<a.size();i++) a[i] = i;

   // Making sure we have the correct size now
   cout << "Verifying resize\n";

   num = a.size();
   if(num==10) cout << "Resize verified\n";
   else cout << "Failure in resize check!\n";

   // Lets try printing everything out with the for_each
   cout << "Verifying each element by printing, you should see number 0 to 9\n";

   for(i=0;i<a.size();i++) cout << a[i] << ",";
   cout << endl;

   // Lets try removing the last element
   cout << "Lets resizing to remove the last element and print again\n";

   a.resize(9);
   for(i=0;i<a.size();i++) cout << a[i] << ",";
   cout << endl;

   // Lets look at some elements with operator[]
   cout << "Lets look at some elements with operator[]\n";

   cout << "a[1] is " << a[1] << endl;

   // Lets modify some elements with operator[]
   cout << "Lets modify some elements with operator[]\n";
   cout << "Setting a[1] to -1.0" << endl;

   a[1] = -1.0;
   cout << "a[1] is now " << a[1] << endl;

   // Testing the clear
   cout << "Verifying the clear\n";
   a.clear();

   num = a.number();
   if(num==0) cout << "Clear number verified\n";
   else cout << "Failure of clear!\n";

   // Checking the appending stuff
   cout << "Doing 10 appends to the vector\n";

   for(i=0;i<10;i++)
      {
      a.append(i);
      }

   // Making sure we have the correct size now
   cout << "Verifying number after 10 appends\n";

   num = a.number();
   if(num==10) cout << "Appending number verified\n";
   else cout << "Failure in appending number check!\n";

   // Lets try printing everything out with the for_each
   cout << "Verifying for_each by printing, you should see number 0 to 9\n";

   a.for_each(dprint,NULL);
   cout << endl;

   // Lets try removing the last element
   cout << "Lets remove the last element and print again\n";

   a.remove();
   a.for_each(dprint,NULL);
   cout << endl;

   // Lets try removing the secont to last element
   cout << "Lets remove the second to last element and print again\n";

   num = a.number();
   a.remove(num-2);
   a.for_each(dprint,NULL);
   cout << endl;

   // Lets find the first object greater than 5
   cout << "Lets find the first object greater than 5\n";
   cout << "using a simple function with no arguments\n";

   double *gtfive = a.first_that(more_than_5,NULL);

   if(gtfive!=NULL) cout << "First element < 5 is " << *gtfive << endl;
   else cout << "Failure to find first element!\n";

   // Lets find the last object greater than 4
   cout << "Lets find the last object less than 4\n";
   cout << "using a simple function passing the 4 as an argument\n";

   double comparison = 4.0;
   double *ltfour = a.last_that(less_than,&comparison);

   if(ltfour!=NULL) cout << "Last element < 4 is " << *ltfour << endl;
   else cout << "Failure to find last element!\n";

   // Lets check that the chunk size works properly
   cout << "Lets check that the chunk size works properly\n";
   cout << "First clear the vector, and then set the chunk size to 10\n";

   a.clear();
   a.set_chunk(10);

   cout << "Now append 20 things to force it to reallocate.  We can then\n";
   cout << "remember the pointer and append 15 more.  The pointer should change.\n";

   for(i=0;i<20;i++) a.append(1.0);
   double* first_data = a;

   for(i=0;i<15;i++) a.append(1.0);
   double* second_data = a;

   cout << "Now appending one more should not cause reallocation.\n";

   a.append(1.0);
   double* third_data = a;

   if((first_data!=second_data)&&(third_data==second_data)) cout << "Chunk size seems to be working ok\n";
   else cout << "Failure of chunk size verification!";
   }


